// Blogosphere Browser 0.1
// Patrick Meister, Juni 2006


class Posts {
  ArrayList postListe;                     // Liste mit Post Objekten
  ArrayList springListe;                   // Liste mit den Springs der Abstnde
  ArrayList attractionListe;               // Liste mit den Attractions der Abstnde
  Particle ersterAnker;                    // Erster Fixierpunkt, rahmt die Post-Partikel ein
  Particle letzterAnker;                   // Zweiter Fixierpunkt
  Vector3D systemStart;                    // aktuelle Start-Position des Systems
  Vector3D systemEnd;                      // aktuelle End-Position des Systems
  Vector3D sollSystemStart;                // Soll-Start-Position des Systems
  Vector3D sollSystemEnd;                  // Soll-End-Position des systems

  float springLength = 0.1;    // 0.1
  float springStrength = 1.7;  // 1.7
  float attractionStrength = 50; // 50
  float springDamping = 0.1;    // 1.7

  int anzahlPosts;                         // Anzahl Posts in diesem Partikel-System      
  int partikelStartBestand;                // Anzahl der Partikel, als dieses Posts-Objekt erstellt wurde
  boolean baumArt;                         // Art dieses Baums: true= kinder, false = eltern
  int anzahlOffenePosts;                   // Speichert die Anzahl offene Posts in diesem Strang.

  // CONSTRUCTOR
  // Richtet das Posts-Objekt ein
  Posts(Vector3D _systemStart, Vector3D _systemEnd, int _anzahlPosts, boolean _baumArt) {
    sollSystemStart = _systemStart;
    sollSystemEnd = _systemEnd;
    systemStart = _systemStart;
    systemEnd = _systemEnd;
    anzahlPosts = _anzahlPosts;
    baumArt = _baumArt;
    partikelStartBestand = physics.numberOfParticles();
     anzahlOffenePosts = 0;

    postListe = new ArrayList();
    springListe = new ArrayList();
    attractionListe = new ArrayList();
    ankerErstellen();
    postsErstellen();
    ankerVerbinden();
    
  }

  // LOOP
  void run() {
    // Jedes Post-objekt ankicken
    //println(anzahlOffenePosts);
    
    
    for (int i = 0; i < postListe.size(); i++) {
      Post p = (Post) postListe.get(i);
      p.run();    
    }
     
   Vector3D start = new Vector3D();
   Vector3D end = new Vector3D();
  
   if (baumArt) {
     // ich bin ein Kinder-baum, 
     // der endpunkt muss also verschoben werden
     float sollX = sollSystemStart.x+anzahlPosts*25+(anzahlOffenePosts)*200;
     float futureX = letzterAnker.position().x() + (sollX-letzterAnker.position().x())/30;
     //println(futureX);
     start.setXYZ(sollSystemStart.x, sollSystemStart.y, 0);
     end.setXYZ(futureX, sollSystemStart.y, 0);
     fill(0,0,200);
     //ellipse(ersterAnker.position().x(), ersterAnker.position().y(), 20, 20);  
     //ellipse(letzterAnker.position().x(), letzterAnker.position().y(), 20, 20);
   } else {
     // ich bin ein elternbaum
     // der startpunkt muss also verschoben werden
     float sollX = sollSystemEnd.x-anzahlPosts*25-(anzahlOffenePosts)*200;
     float futureX = ersterAnker.position().x() - (ersterAnker.position().x()-sollX)/30;
     //println(futureX);
     start.setXYZ(futureX, sollSystemEnd.y, 0);
     end.setXYZ(sollSystemEnd.x, sollSystemEnd.y, 0);
     fill(0,0,200);
     //ellipse(ersterAnker.position().x(), ersterAnker.position().y(), 20, 20);  
   }
   
   ankerVerschieben(start, end);
     
     
  }


  void ankerErstellen() {
    // Die beiden usseren, nicht sichtbaren Partikel erstellen.
    // Sie sind fixiert, damit sie die Post-Partikel halten.
    ersterAnker = physics.makeParticle();
    letzterAnker = physics.makeParticle();
    ersterAnker.moveTo(systemStart.x, systemStart.y, systemStart.z);
    letzterAnker.moveTo(systemEnd.x, systemEnd.y, systemEnd.z);
    ersterAnker.makeFixed();
    letzterAnker.makeFixed();  
  }


  void postsErstellen() {
    // Erstellt Posts
    // zwischen den Ankern
    
    for (int i = 0; i < anzahlPosts; i++) {
      Vector3D startPunkt = new Vector3D();
      float neuX;
      if (baumArt) {
        // bin ein kinderbaum
        neuX =  sollSystemStart.x +((sollSystemEnd.x-sollSystemStart.x)/anzahlPosts*(i));
      } else {
        neuX =  sollSystemEnd.x - anzahlPosts*25 - ((sollSystemEnd.x-sollSystemStart.x)/anzahlPosts*(i));
      }
      startPunkt.setXYZ(neuX, mouseY/smoother.z() - smoother.y(), 0);
      fill(0,200,0);
      //ellipse(startPunkt.x, startPunkt.y, 10, 10);
      Post neuerPost = new Post(startPunkt, baumArt, i, anzahlPosts);
      postListe.add(neuerPost);
      if (i > 0) {
        Particle p1 = physics.getParticle(physics.numberOfParticles()-2);
        Particle p2 = physics.getParticle(physics.numberOfParticles()-3);
        abstandSpring(p1, p2);
        //abstandAttraction(p1, p2);
      }
      neuerPost.postsRef(this);

    }
  }


  void ankerVerbinden() {
    Particle p1 = physics.getParticle(partikelStartBestand);
    Particle p2 = physics.getParticle(partikelStartBestand+2);
    Particle p3 = physics.getParticle(physics.numberOfParticles()-1);
    Particle p4 = physics.getParticle(partikelStartBestand+1);
    abstandSpring(p1, p2);
    abstandSpring(p3, p4);
  }

  void abstandAttraction( Particle p, Particle r )
  {
    for ( int i = 0; i < physics.numberOfParticles(); ++i )
    {
      Particle q = physics.getParticle( i );
      if ( p != q && p != r ) {
        Attraction a = physics.makeAttraction( p, q, -attractionStrength, 10 );
        attractionListe.add(a);
      }
    }
  }

  void abstandSpring( Particle a, Particle b )
  {
    Spring s = physics.makeSpring( a, b, springStrength, springDamping, springLength );
    springListe.add(s);
  }

  public void ankerVerschieben(Vector3D _start, Vector3D _end) {

    systemStart = _start.copy(); //.setXYZ(systemStart.x, mouseY, 0);
    systemEnd = _end.copy(); //.setXYZ(systemEnd.x, mouseY, 0);
    ersterAnker.moveTo(systemStart.x, systemStart.y, systemStart.z);
    letzterAnker.moveTo(systemEnd.x, systemEnd.y, systemEnd.z);  
  }

  public void updateOffenePosts(int _veraenderung) {
   //wird von den eigenen posts aufgerufen, wenn sie sich ffnen oder schliessen 
   anzahlOffenePosts += _veraenderung;
   println(anzahlOffenePosts);

   
   
  }
  
  void kill()
    // killt alle Posts
  {
    for (int i = 0; i < postListe.size(); i++) {
      Post p = (Post) postListe.get(i);
      p.kill();    
    }

  }

}
